{
  "AUX__Mood=Ind|Number=Sing|Person=3|Tense=Past|VerbForm=Fin": {
    "freq": 865,
    "morph": "Mood=Ind|Number=Sing|Person=3|Tense=Past|VerbForm=Fin",
    "pos": "AUX"
  },
  "PUNCT__PunctSide=Fin|PunctType=Brck": {
    "freq": 1476,
    "morph": "PunctSide=Fin|PunctType=Brck",
    "pos": "PUNCT"
  },
  "VERB__Mood=Ind|Number=Sing|Person=3|Tense=Past|VerbForm=Fin": {
    "freq": 7033,
    "morph": "Mood=Ind|Number=Sing|Person=3|Tense=Past|VerbForm=Fin",
    "pos": "VERB"
  },
  "PRON__Number=Sing|Person=2|PronType=Prs": {
    "freq": 132,
    "morph": "Number=Sing|Person=2|PronType=Prs",
    "pos": "PRON"
  },
  "PRON": {
    "pos": "PRON"
  },
  "VERB__Mood=Ind|Number=Sing|Person=1|Tense=Pres|VerbForm=Fin": {
    "freq": 525,
    "morph": "Mood=Ind|Number=Sing|Person=1|Tense=Pres|VerbForm=Fin",
    "pos": "VERB"
  },
  "SYM__NumForm=Digit|NumType=Frac": {
    "freq": 236,
    "morph": "NumForm=Digit|NumType=Frac",
    "pos": "SYM"
  },
  "ADJ___": {
    "freq": 515,
    "morph": "_",
    "pos": "ADJ"
  },
  "PRON__Person=3": {
    "freq": 3185,
    "morph": "Person=3",
    "pos": "PRON"
  },
  "PRON__Case=Acc|Gender=Masc|Number=Plur|Person=3|PronType=Prs": {
    "freq": 104,
    "morph": "Case=Acc|Gender=Masc|Number=Plur|Person=3|PronType=Prs",
    "pos": "PRON"
  },
  "DET__Number=Sing|Number[psor]=Sing|Person=1|Poss=Yes|PronType=Prs": {
    "freq": 148,
    "morph": "Number=Sing|Number[psor]=Sing|Person=1|Poss=Yes|PronType=Prs",
    "pos": "DET"
  },
  "CONJ": {
    "pos": "CONJ"
  },
  "PUNCT__PunctType=Comm": {
    "freq": 24475,
    "morph": "PunctType=Comm",
    "pos": "PUNCT"
  },
  "ADV": {
    "pos": "ADV"
  },
  "ADV__AdpType=Prep": {
    "freq": 161,
    "morph": "AdpType=Prep",
    "pos": "ADV"
  },
  "ADJ__Number=Plur": {
    "freq": 2617,
    "morph": "Number=Plur",
    "pos": "ADJ"
  },
  "AUX__Mood=Sub|Number=Sing|Person=3|Tense=Imp|VerbForm=Fin": {
    "freq": 149,
    "morph": "Mood=Sub|Number=Sing|Person=3|Tense=Imp|VerbForm=Fin",
    "pos": "AUX"
  },
  "ADJ__Gender=Masc|Number=Sing|NumType=Ord": {
    "freq": 654,
    "morph": "Gender=Masc|Number=Sing|NumType=Ord",
    "pos": "ADJ"
  },
  "AUX__Mood=Cnd|Number=Sing|Person=3|VerbForm=Fin": {
    "freq": 272,
    "morph": "Mood=Cnd|Number=Sing|Person=3|VerbForm=Fin",
    "pos": "AUX"
  },
  "AUX__Mood=Ind|Number=Sing|Person=3|Tense=Fut|VerbForm=Fin": {
    "freq": 388,
    "morph": "Mood=Ind|Number=Sing|Person=3|Tense=Fut|VerbForm=Fin",
    "pos": "AUX"
  },
  "ADJ__Gender=Masc|Number=Plur": {
    "freq": 1995,
    "morph": "Gender=Masc|Number=Plur",
    "pos": "ADJ"
  },
  "DET": {
    "pos": "DET"
  },
  "VERB__VerbForm=Inf": {
    "freq": 8204,
    "morph": "VerbForm=Inf",
    "pos": "VERB"
  },
  "DET__Definite=Def|Gender=Fem|Number=Plur|PronType=Art": {
    "freq": 4275,
    "morph": "Definite=Def|Gender=Fem|Number=Plur|PronType=Art",
    "pos": "DET"
  },
  "VERB__Mood=Ind|Number=Plur|Person=3|Tense=Imp|VerbForm=Fin": {
    "freq": 495,
    "morph": "Mood=Ind|Number=Plur|Person=3|Tense=Imp|VerbForm=Fin",
    "pos": "VERB"
  },
  "DET__Definite=Def|Gender=Masc|Number=Plur|PronType=Art": {
    "freq": 6951,
    "morph": "Definite=Def|Gender=Masc|Number=Plur|PronType=Art",
    "pos": "DET"
  },
  "PRON___": {
    "freq": 1871,
    "morph": "_",
    "pos": "PRON"
  },
  "DET__Definite=Ind|Gender=Masc|Number=Plur|PronType=Art": {
    "freq": 113,
    "morph": "Definite=Ind|Gender=Masc|Number=Plur|PronType=Art",
    "pos": "DET"
  },
  "NOUN__Number=Sing": {
    "freq": 1977,
    "morph": "Number=Sing",
    "pos": "NOUN"
  },
  "ADJ__Gender=Fem|Number=Sing|NumType=Ord": {
    "freq": 568,
    "morph": "Gender=Fem|Number=Sing|NumType=Ord",
    "pos": "ADJ"
  },
  "NOUN__Gender=Masc|Number=Sing": {
    "freq": 25557,
    "morph": "Gender=Masc|Number=Sing",
    "pos": "NOUN"
  },
  "PART": {
    "pos": "PART"
  },
  "ADJ__Number=Sing": {
    "freq": 6619,
    "morph": "Number=Sing",
    "pos": "ADJ"
  },
  "NUM": {
    "pos": "NUM"
  },
  "DET__Number=Sing|PronType=Ind": {
    "freq": 309,
    "morph": "Number=Sing|PronType=Ind",
    "pos": "DET"
  },
  "ADJ__Gender=Fem|Number=Sing|VerbForm=Part": {
    "freq": 1387,
    "morph": "Gender=Fem|Number=Sing|VerbForm=Part",
    "pos": "ADJ"
  },
  "VERB__Mood=Sub|Number=Sing|Person=3|Tense=Imp|VerbForm=Fin": {
    "freq": 272,
    "morph": "Mood=Sub|Number=Sing|Person=3|Tense=Imp|VerbForm=Fin",
    "pos": "VERB"
  },
  "VERB__Mood=Ind|Number=Plur|Person=3|Tense=Past|VerbForm=Fin": {
    "freq": 1574,
    "morph": "Mood=Ind|Number=Plur|Person=3|Tense=Past|VerbForm=Fin",
    "pos": "VERB"
  },
  "PRON__Gender=Masc|Number=Sing|PronType=Dem": {
    "freq": 115,
    "morph": "Gender=Masc|Number=Sing|PronType=Dem",
    "pos": "PRON"
  },
  "ADP": {
    "pos": "ADP"
  },
  "NOUN__AdvType=Tim": {
    "freq": 1504,
    "morph": "AdvType=Tim",
    "pos": "NOUN"
  },
  "AUX__Mood=Ind|Number=Plur|Person=3|Tense=Fut|VerbForm=Fin": {
    "freq": 130,
    "morph": "Mood=Ind|Number=Plur|Person=3|Tense=Fut|VerbForm=Fin",
    "pos": "AUX"
  },
  "PRON__Case=Nom|Number=Sing|Person=1|PronType=Prs": {
    "freq": 115,
    "morph": "Case=Nom|Number=Sing|Person=1|PronType=Prs",
    "pos": "PRON"
  },
  "PUNCT__PunctType=Semi": {
    "freq": 259,
    "morph": "PunctType=Semi",
    "pos": "PUNCT"
  },
  "PUNCT__PunctSide=Ini|PunctType=Qest": {
    "freq": 206,
    "morph": "PunctSide=Ini|PunctType=Qest",
    "pos": "PUNCT"
  },
  "PRON__Case=Dat|Number=Sing|Person=3|PronType=Prs": {
    "freq": 754,
    "morph": "Case=Dat|Number=Sing|Person=3|PronType=Prs",
    "pos": "PRON"
  },
  "PRON__Case=Acc|Gender=Masc|Number=Sing|Person=3|PronType=Prs": {
    "freq": 624,
    "morph": "Case=Acc|Gender=Masc|Number=Sing|Person=3|PronType=Prs",
    "pos": "PRON"
  },
  "NUM__NumForm=Digit": {
    "freq": 2979,
    "morph": "NumForm=Digit",
    "pos": "NUM"
  },
  "PUNCT__PunctType=Colo": {
    "freq": 638,
    "morph": "PunctType=Colo",
    "pos": "PUNCT"
  },
  "PROPN": {
    "pos": "PROPN"
  },
  "X": {
    "pos": "X"
  },
  "NOUN__NumForm=Digit": {
    "freq": 555,
    "morph": "NumForm=Digit",
    "pos": "NOUN"
  },
  "VERB__Gender=Masc|Number=Sing|Tense=Past|VerbForm=Part": {
    "freq": 3297,
    "morph": "Gender=Masc|Number=Sing|Tense=Past|VerbForm=Part",
    "pos": "VERB"
  },
  "ADJ__Gender=Masc|Number=Plur|NumType=Ord": {
    "freq": 227,
    "morph": "Gender=Masc|Number=Plur|NumType=Ord",
    "pos": "ADJ"
  },
  "PRON__Gender=Masc|Number=Sing|Person=3|PronType=Prs": {
    "freq": 205,
    "morph": "Gender=Masc|Number=Sing|Person=3|PronType=Prs",
    "pos": "PRON"
  },
  "NOUN__Number=Plur": {
    "freq": 1463,
    "morph": "Number=Plur",
    "pos": "NOUN"
  },
  "DET__Number=Sing|Person=3|Poss=Yes|PronType=Prs": {
    "freq": 2909,
    "morph": "Number=Sing|Person=3|Poss=Yes|PronType=Prs",
    "pos": "DET"
  },
  "VERB__VerbForm=Ger": {
    "freq": 994,
    "morph": "VerbForm=Ger",
    "pos": "VERB"
  },
  "INTJ": {
    "pos": "INTJ"
  },
  "VERB__Mood=Ind|Number=Plur|Person=3|Tense=Fut|VerbForm=Fin": {
    "freq": 398,
    "morph": "Mood=Ind|Number=Plur|Person=3|Tense=Fut|VerbForm=Fin",
    "pos": "VERB"
  },
  "AUX__Mood=Ind|Number=Plur|Person=3|Tense=Pres|VerbForm=Fin": {
    "freq": 1403,
    "morph": "Mood=Ind|Number=Plur|Person=3|Tense=Pres|VerbForm=Fin",
    "pos": "AUX"
  },
  "PRON__Number=Plur|Person=1|PronType=Prs": {
    "freq": 264,
    "morph": "Number=Plur|Person=1|PronType=Prs",
    "pos": "PRON"
  },
  "ADV__Negative=Neg": {
    "freq": 2960,
    "morph": "Negative=Neg",
    "pos": "ADV"
  },
  "VERB__Mood=Ind|Number=Plur|Person=3|Tense=Pres|VerbForm=Fin": {
    "freq": 2488,
    "morph": "Mood=Ind|Number=Plur|Person=3|Tense=Pres|VerbForm=Fin",
    "pos": "VERB"
  },
  "DET__Gender=Masc|Number=Sing|PronType=Ind": {
    "freq": 855,
    "morph": "Gender=Masc|Number=Sing|PronType=Ind",
    "pos": "DET"
  },
  "VERB__Mood=Sub|Number=Plur|Person=3|Tense=Pres|VerbForm=Fin": {
    "freq": 408,
    "morph": "Mood=Sub|Number=Plur|Person=3|Tense=Pres|VerbForm=Fin",
    "pos": "VERB"
  },
  "PRON__Gender=Fem|Number=Sing|PronType=Ind": {
    "freq": 237,
    "morph": "Gender=Fem|Number=Sing|PronType=Ind",
    "pos": "PRON"
  },
  "DET__Gender=Fem|Number=Plur|PronType=Ind": {
    "freq": 592,
    "morph": "Gender=Fem|Number=Plur|PronType=Ind",
    "pos": "DET"
  },
  "ADJ__Gender=Fem|Number=Plur|VerbForm=Part": {
    "freq": 614,
    "morph": "Gender=Fem|Number=Plur|VerbForm=Part",
    "pos": "ADJ"
  },
  "DET__Gender=Fem|Number=Sing|PronType=Dem": {
    "freq": 808,
    "morph": "Gender=Fem|Number=Sing|PronType=Dem",
    "pos": "DET"
  },
  "DET__Gender=Fem|Number=Sing|PronType=Ind": {
    "freq": 613,
    "morph": "Gender=Fem|Number=Sing|PronType=Ind",
    "pos": "DET"
  },
  "DET__Definite=Ind|Gender=Masc|Number=Sing|PronType=Art": {
    "freq": 4277,
    "morph": "Definite=Ind|Gender=Masc|Number=Sing|PronType=Art",
    "pos": "DET"
  },
  "VERB__Mood=Sub|Number=Sing|Person=3|Tense=Pres|VerbForm=Fin": {
    "freq": 788,
    "morph": "Mood=Sub|Number=Sing|Person=3|Tense=Pres|VerbForm=Fin",
    "pos": "VERB"
  },
  "NOUN__Gender=Fem": {
    "freq": 145,
    "morph": "Gender=Fem",
    "pos": "NOUN"
  },
  "PRON__Gender=Fem|Number=Plur|PronType=Ind": {
    "freq": 127,
    "morph": "Gender=Fem|Number=Plur|PronType=Ind",
    "pos": "PRON"
  },
  "AUX__Mood=Ind|Number=Sing|Person=3|Tense=Imp|VerbForm=Fin": {
    "freq": 729,
    "morph": "Mood=Ind|Number=Sing|Person=3|Tense=Imp|VerbForm=Fin",
    "pos": "AUX"
  },
  "VERB__Mood=Ind|Number=Sing|Person=3|Tense=Imp|VerbForm=Fin": {
    "freq": 1223,
    "morph": "Mood=Ind|Number=Sing|Person=3|Tense=Imp|VerbForm=Fin",
    "pos": "VERB"
  },
  "AUX__Mood=Ind|Number=Sing|Person=1|Tense=Pres|VerbForm=Fin": {
    "freq": 164,
    "morph": "Mood=Ind|Number=Sing|Person=1|Tense=Pres|VerbForm=Fin",
    "pos": "AUX"
  },
  "PRON__PronType=Rel": {
    "freq": 7301,
    "morph": "PronType=Rel",
    "pos": "PRON"
  },
  "DET__Definite=Def|Number=Sing|PronType=Art": {
    "freq": 928,
    "morph": "Definite=Def|Number=Sing|PronType=Art",
    "pos": "DET"
  },
  "ADV___": {
    "freq": 11334,
    "morph": "_",
    "pos": "ADV"
  },
  "ADJ": {
    "pos": "ADJ"
  },
  "AUX__VerbForm=Ger": {
    "freq": 154,
    "morph": "VerbForm=Ger",
    "pos": "AUX"
  },
  "PRON__Number=Sing|PronType=Int": {
    "freq": 201,
    "morph": "Number=Sing|PronType=Int",
    "pos": "PRON"
  },
  "VERB__Mood=Ind|Number=Sing|Person=3|Tense=Fut|VerbForm=Fin": {
    "freq": 1236,
    "morph": "Mood=Ind|Number=Sing|Person=3|Tense=Fut|VerbForm=Fin",
    "pos": "VERB"
  },
  "NOUN__Gender=Masc|Number=Plur": {
    "freq": 12310,
    "morph": "Gender=Masc|Number=Plur",
    "pos": "NOUN"
  },
  "NOUN__Gender=Fem|Number=Plur": {
    "freq": 8612,
    "morph": "Gender=Fem|Number=Plur",
    "pos": "NOUN"
  },
  "VERB__Mood=Ind|Number=Sing|Person=3|Tense=Pres|VerbForm=Fin": {
    "freq": 6343,
    "morph": "Mood=Ind|Number=Sing|Person=3|Tense=Pres|VerbForm=Fin",
    "pos": "VERB"
  },
  "PRON__Gender=Masc|Number=Plur|PronType=Ind": {
    "freq": 460,
    "morph": "Gender=Masc|Number=Plur|PronType=Ind",
    "pos": "PRON"
  },
  "VERB__Mood=Sub|Number=Plur|Person=3|Tense=Imp|VerbForm=Fin": {
    "freq": 100,
    "morph": "Mood=Sub|Number=Plur|Person=3|Tense=Imp|VerbForm=Fin",
    "pos": "VERB"
  },
  "PUNCT__PunctSide=Ini|PunctType=Brck": {
    "freq": 1482,
    "morph": "PunctSide=Ini|PunctType=Brck",
    "pos": "PUNCT"
  },
  "PRON__Gender=Masc|Number=Sing|PronType=Tot": {
    "freq": 111,
    "morph": "Gender=Masc|Number=Sing|PronType=Tot",
    "pos": "PRON"
  },
  "SCONJ": {
    "pos": "SCONJ"
  },
  "AUX__VerbForm=Inf": {
    "freq": 1495,
    "morph": "VerbForm=Inf",
    "pos": "AUX"
  },
  "AUX__Mood=Ind|Number=Sing|Person=3|Tense=Pres|VerbForm=Fin": {
    "freq": 5227,
    "morph": "Mood=Ind|Number=Sing|Person=3|Tense=Pres|VerbForm=Fin",
    "pos": "AUX"
  },
  "ADJ__AdpType=Prep": {
    "freq": 124,
    "morph": "AdpType=Prep",
    "pos": "ADJ"
  },
  "PRON__Gender=Masc|Number=Sing|PronType=Ind": {
    "freq": 624,
    "morph": "Gender=Masc|Number=Sing|PronType=Ind",
    "pos": "PRON"
  },
  "DET__Gender=Masc|Number=Plur|PronType=Dem": {
    "freq": 269,
    "morph": "Gender=Masc|Number=Plur|PronType=Dem",
    "pos": "DET"
  },
  "ADJ__Gender=Fem|Number=Plur": {
    "freq": 1612,
    "morph": "Gender=Fem|Number=Plur",
    "pos": "ADJ"
  },
  "NUM__Gender=Masc|Number=Plur|NumType=Card": {
    "freq": 104,
    "morph": "Gender=Masc|Number=Plur|NumType=Card",
    "pos": "NUM"
  },
  "NUM__NumType=Card": {
    "freq": 533,
    "morph": "NumType=Card",
    "pos": "NUM"
  },
  "SCONJ___": {
    "freq": 10129,
    "morph": "_",
    "pos": "SCONJ"
  },
  "PRON__Number=Sing|PronType=Rel": {
    "freq": 318,
    "morph": "Number=Sing|PronType=Rel",
    "pos": "PRON"
  },
  "VERB__Mood=Cnd|Number=Sing|Person=3|VerbForm=Fin": {
    "freq": 253,
    "morph": "Mood=Cnd|Number=Sing|Person=3|VerbForm=Fin",
    "pos": "VERB"
  },
  "NOUN": {
    "pos": "NOUN"
  },
  "NOUN__Gender=Masc": {
    "freq": 153,
    "morph": "Gender=Masc",
    "pos": "NOUN"
  },
  "DET__Definite=Ind|Gender=Fem|Number=Sing|PronType=Art": {
    "freq": 3087,
    "morph": "Definite=Ind|Gender=Fem|Number=Sing|PronType=Art",
    "pos": "DET"
  },
  "ADJ__Gender=Masc|Number=Plur|VerbForm=Part": {
    "freq": 997,
    "morph": "Gender=Masc|Number=Plur|VerbForm=Part",
    "pos": "ADJ"
  },
  "PRON__Number=Sing|PronType=Dem": {
    "freq": 302,
    "morph": "Number=Sing|PronType=Dem",
    "pos": "PRON"
  },
  "PRON__Number=Sing|Person=3|PronType=Prs": {
    "freq": 116,
    "morph": "Number=Sing|Person=3|PronType=Prs",
    "pos": "PRON"
  },
  "PRON__Case=Acc|Gender=Fem|Number=Sing|Person=3|PronType=Prs": {
    "freq": 173,
    "morph": "Case=Acc|Gender=Fem|Number=Sing|Person=3|PronType=Prs",
    "pos": "PRON"
  },
  "PUNCT": {
    "pos": "PUNCT"
  },
  "DET__Gender=Masc|Number=Sing|PronType=Dem": {
    "freq": 962,
    "morph": "Gender=Masc|Number=Sing|PronType=Dem",
    "pos": "DET"
  },
  "PRON__Number=Plur|PronType=Rel": {
    "freq": 102,
    "morph": "Number=Plur|PronType=Rel",
    "pos": "PRON"
  },
  "ADJ__Gender=Masc|Number=Sing": {
    "freq": 5136,
    "morph": "Gender=Masc|Number=Sing",
    "pos": "ADJ"
  },
  "DET__Definite=Def|Gender=Masc|Number=Sing|PronType=Art": {
    "freq": 22962,
    "morph": "Definite=Def|Gender=Masc|Number=Sing|PronType=Art",
    "pos": "DET"
  },
  "AUX__Mood=Sub|Number=Plur|Person=3|Tense=Pres|VerbForm=Fin": {
    "freq": 107,
    "morph": "Mood=Sub|Number=Plur|Person=3|Tense=Pres|VerbForm=Fin",
    "pos": "AUX"
  },
  "PRON__Case=Dat|Number=Plur|Person=3|PronType=Prs": {
    "freq": 220,
    "morph": "Case=Dat|Number=Plur|Person=3|PronType=Prs",
    "pos": "PRON"
  },
  "VERB__Gender=Fem|Number=Sing|Tense=Past|VerbForm=Part": {
    "freq": 206,
    "morph": "Gender=Fem|Number=Sing|Tense=Past|VerbForm=Part",
    "pos": "VERB"
  },
  "DET__Number=Plur|Person=3|Poss=Yes|PronType=Prs": {
    "freq": 1021,
    "morph": "Number=Plur|Person=3|Poss=Yes|PronType=Prs",
    "pos": "DET"
  },
  "ADJ__Gender=Fem|Number=Plur|NumType=Ord": {
    "freq": 101,
    "morph": "Gender=Fem|Number=Plur|NumType=Ord",
    "pos": "ADJ"
  },
  "PRON__PronType=Int": {
    "freq": 137,
    "morph": "PronType=Int",
    "pos": "PRON"
  },
  "ADP__AdpType=Prep": {
    "freq": 71133,
    "morph": "AdpType=Prep",
    "pos": "ADP"
  },
  "DET__Gender=Masc|Number=Plur|PronType=Ind": {
    "freq": 904,
    "morph": "Gender=Masc|Number=Plur|PronType=Ind",
    "pos": "DET"
  },
  "AUX__Mood=Sub|Number=Sing|Person=3|Tense=Pres|VerbForm=Fin": {
    "freq": 299,
    "morph": "Mood=Sub|Number=Sing|Person=3|Tense=Pres|VerbForm=Fin",
    "pos": "AUX"
  },
  "DET__Gender=Fem|Number=Plur|PronType=Dem": {
    "freq": 188,
    "morph": "Gender=Fem|Number=Plur|PronType=Dem",
    "pos": "DET"
  },
  "NUM__NumForm=Digit|NumType=Card": {
    "freq": 1108,
    "morph": "NumForm=Digit|NumType=Card",
    "pos": "NUM"
  },
  "PUNCT__PunctType=Quot": {
    "freq": 7380,
    "morph": "PunctType=Quot",
    "pos": "PUNCT"
  },
  "VERB__Gender=Masc|Number=Plur|Tense=Past|VerbForm=Part": {
    "freq": 184,
    "morph": "Gender=Masc|Number=Plur|Tense=Past|VerbForm=Part",
    "pos": "VERB"
  },
  "PUNCT__PunctType=Dash": {
    "freq": 2345,
    "morph": "PunctType=Dash",
    "pos": "PUNCT"
  },
  "ADJ__Gender=Fem|Number=Sing": {
    "freq": 3935,
    "morph": "Gender=Fem|Number=Sing",
    "pos": "ADJ"
  },
  "AUX__Mood=Ind|Number=Plur|Person=1|Tense=Pres|VerbForm=Fin": {
    "freq": 215,
    "morph": "Mood=Ind|Number=Plur|Person=1|Tense=Pres|VerbForm=Fin",
    "pos": "AUX"
  },
  "AUX__Mood=Ind|Number=Plur|Person=3|Tense=Past|VerbForm=Fin": {
    "freq": 218,
    "morph": "Mood=Ind|Number=Plur|Person=3|Tense=Past|VerbForm=Fin",
    "pos": "AUX"
  },
  "PROPN___": {
    "freq": 34454,
    "morph": "_",
    "pos": "PROPN"
  },
  "PRON__Number=Sing|PronType=Ind": {
    "freq": 421,
    "morph": "Number=Sing|PronType=Ind",
    "pos": "PRON"
  },
  "VERB__Mood=Ind|Number=Plur|Person=1|Tense=Pres|VerbForm=Fin": {
    "freq": 359,
    "morph": "Mood=Ind|Number=Plur|Person=1|Tense=Pres|VerbForm=Fin",
    "pos": "VERB"
  },
  "PUNCT__PunctSide=Fin|PunctType=Qest": {
    "freq": 312,
    "morph": "PunctSide=Fin|PunctType=Qest",
    "pos": "PUNCT"
  },
  "PRON__Number=Sing|Person=1|PronType=Prs": {
    "freq": 298,
    "morph": "Number=Sing|Person=1|PronType=Prs",
    "pos": "PRON"
  },
  "PART__Negative=Neg": {
    "freq": 122,
    "morph": "Negative=Neg",
    "pos": "PART"
  },
  "PRON__Gender=Masc|Number=Plur|Person=3|PronType=Prs": {
    "freq": 176,
    "morph": "Gender=Masc|Number=Plur|Person=3|PronType=Prs",
    "pos": "PRON"
  },
  "NOUN__Gender=Fem|Number=Sing": {
    "freq": 24416,
    "morph": "Gender=Fem|Number=Sing",
    "pos": "NOUN"
  },
  "ADJ__Gender=Masc|Number=Sing|VerbForm=Part": {
    "freq": 2297,
    "morph": "Gender=Masc|Number=Sing|VerbForm=Part",
    "pos": "ADJ"
  },
  "CONJ___": {
    "freq": 12225,
    "morph": "_",
    "pos": "CONJ"
  },
  "NUM__Number=Plur|NumType=Card": {
    "freq": 2057,
    "morph": "Number=Plur|NumType=Card",
    "pos": "NUM"
  },
  "NOUN___": {
    "freq": 4829,
    "morph": "_",
    "pos": "NOUN"
  },
  "VERB": {
    "pos": "VERB"
  },
  "DET__Definite=Def|Gender=Fem|Number=Sing|PronType=Art": {
    "freq": 16487,
    "morph": "Definite=Def|Gender=Fem|Number=Sing|PronType=Art",
    "pos": "DET"
  },
  "SYM": {
    "pos": "SYM"
  },
  "VERB__Mood=Ind|Number=Sing|Person=1|Tense=Past|VerbForm=Fin": {
    "freq": 130,
    "morph": "Mood=Ind|Number=Sing|Person=1|Tense=Past|VerbForm=Fin",
    "pos": "VERB"
  },
  "AUX": {
    "pos": "AUX"
  },
  "AUX__Gender=Masc|Number=Sing|Tense=Past|VerbForm=Part": {
    "freq": 494,
    "morph": "Gender=Masc|Number=Sing|Tense=Past|VerbForm=Part",
    "pos": "AUX"
  },
  "AUX__Mood=Ind|Number=Plur|Person=3|Tense=Imp|VerbForm=Fin": {
    "freq": 199,
    "morph": "Mood=Ind|Number=Plur|Person=3|Tense=Imp|VerbForm=Fin",
    "pos": "AUX"
  },
  "VERB__Mood=Imp|Number=Sing|Person=2|VerbForm=Fin": {
    "freq": 100,
    "morph": "Mood=Imp|Number=Sing|Person=2|VerbForm=Fin",
    "pos": "VERB"
  },
  "PUNCT__PunctType=Peri": {
    "freq": 14170,
    "morph": "PunctType=Peri",
    "pos": "PUNCT"
  }
}


# coding: utf8
from __future__ import unicode_literals, print_function
import os
import io
import json
import re
import os.path
import pathlib
import sys

import six
import textwrap

from .attrs import TAG, HEAD, DEP, ENT_IOB, ENT_TYPE

try:
    basestring
except NameError:
    basestring = str


LANGUAGES = {}
_data_path = pathlib.Path(__file__).parent / 'data'


def set_lang_class(name, cls):
    global LANGUAGES
    LANGUAGES[name] = cls


def get_lang_class(name):
    lang = re.split('[^a-zA-Z0-9]', name, 1)[0]
    if lang not in LANGUAGES:
        raise RuntimeError('Language not supported: %s' % lang)
    return LANGUAGES[lang]


def get_data_path(require_exists=True):
    if not require_exists:
        return _data_path
    else:
        return _data_path if _data_path.exists() else None


def set_data_path(path):
    global _data_path
    if isinstance(path, basestring):
        path = pathlib.Path(path)
    _data_path = path


def or_(val1, val2):
    if val1 is not None:
        return val1
    elif callable(val2):
        return val2()
    else:
        return val2


def match_best_version(target_name, target_version, path):
    path = path if not isinstance(path, basestring) else pathlib.Path(path)
    if path is None or not path.exists():
        return None
    matches = []
    for data_name in path.iterdir():
        name, version = split_data_name(data_name.parts[-1])
        if name == target_name and constraint_match(target_version, version):
            matches.append((tuple(float(v) for v in version.split('.')), data_name))
    if matches:
        return pathlib.Path(max(matches)[1])
    else:
        return None


def split_data_name(name):
    return name.split('-', 1) if '-' in name else (name, '')


def constraint_match(constraint_string, version):
    # From http://github.com/spacy-io/sputnik
    if not constraint_string:
        return True

    constraints = [c.strip() for c in constraint_string.split(',') if c.strip()]

    for c in constraints:
        if not re.match(r'[><=][=]?\d+(\.\d+)*', c):
            raise ValueError('invalid constraint: %s' % c)

    return all(semver.match(version, c) for c in constraints)


def read_regex(path):
    path = path if not isinstance(path, basestring) else pathlib.Path(path)
    with path.open() as file_:
        entries = file_.read().split('\n')
    expression = '|'.join(['^' + re.escape(piece) for piece in entries if piece.strip()])
    return re.compile(expression)


def compile_prefix_regex(entries):
    if '(' in entries:
        # Handle deprecated data
        expression = '|'.join(['^' + re.escape(piece) for piece in entries if piece.strip()])
        return re.compile(expression)
    else:
        expression = '|'.join(['^' + piece for piece in entries if piece.strip()])
        return re.compile(expression)


def compile_suffix_regex(entries):
    expression = '|'.join([piece + '$' for piece in entries if piece.strip()])
    return re.compile(expression)


def compile_infix_regex(entries):
    expression = '|'.join([piece for piece in entries if piece.strip()])
    return re.compile(expression)


def normalize_slice(length, start, stop, step=None):
    if not (step is None or step == 1):
        raise ValueError("Stepped slices not supported in Span objects."
                         "Try: list(tokens)[start:stop:step] instead.")
    if start is None:
       start = 0
    elif start < 0:
       start += length
    start = min(length, max(0, start))

    if stop is None:
       stop = length
    elif stop < 0:
       stop += length
    stop = min(length, max(start, stop))

    assert 0 <= start <= stop <= length
    return start, stop


def utf8open(loc, mode='r'):
    return io.open(loc, mode, encoding='utf8')


def check_renamed_kwargs(renamed, kwargs):
    for old, new in renamed.items():
        if old in kwargs:
            raise TypeError("Keyword argument %s now renamed to %s" % (old, new))


def parse_package_meta(package_path, package, require=True):
    location = os.path.join(str(package_path), package, 'meta.json')
    if os.path.isfile(location):
        with io.open(location, encoding='utf8') as f:
            meta = json.load(f)
            return meta
    elif require:
        raise IOError("Could not read meta.json from %s" % location)
    else:
        return None


def print_table(data, **kwargs):
    """Print data in table format. Can either take a list of tuples or a
    dictionary, which will be converted to a list of tuples."""

    if type(data) == dict:
        data = list(data.items())

    tpl_msg = '\n{msg}\n'
    tpl_title = '\n    \033[93m{msg}\033[0m'
    tpl_row ="    {:<15}" * len(data[0])
    table = '\n'.join([tpl_row.format(l, v) for l, v in data])

    if 'title' in kwargs and kwargs['title']:
        print(tpl_title.format(msg=kwargs['title']))

    print(tpl_msg.format(msg=table))


def print_markdown(data, **kwargs):
    """Print listed data in GitHub-flavoured Markdown format so it can be
    copy-pasted into issues. Can either take a list of tuples or a dictionary,
    which will be converted to a list of tuples."""

    def excl_value(value):
        # don't print value if it contains absolute path of directory
        # (i.e. personal info that shouldn't need to be shared)
        # other conditions can be included here if necessary
        if str(pathlib.Path(__file__).parent) in value:
            return True

    if type(data) == dict:
        data = list(data.items())

    tpl_msg = "\n{msg}\n"
    tpl_title = "\n## {msg}"
    tpl_row = "* **{l}:** {v}"
    markdown = '\n'.join([tpl_row.format(l=l, v=v) for l, v in data if not excl_value(v)])

    if 'title' in kwargs and kwargs['title']:
        print(tpl_title.format(msg=kwargs['title']))

    print(tpl_msg.format(msg=markdown))


def print_msg(*text, **kwargs):
    """Print formatted message. Each positional argument is rendered as newline-
    separated paragraph. If kwarg 'title' exist, title is printed above the text
    and highlighted (using ANSI escape sequences manually to avoid unnecessary
    dependency)."""

    message = '\n\n'.join([_wrap_text(t) for t in text])
    tpl_msg = '\n{msg}\n'
    tpl_title = '\n\033[93m{msg}\033[0m'

    if 'title' in kwargs and kwargs['title']:
        title = _wrap_text(kwargs['title'])
        print(tpl_title.format(msg=title))
    print(tpl_msg.format(msg=message))


def _wrap_text(text):
    """Wrap text at given width using textwrap module. Indent should consist of
    spaces. Its length is deducted from wrap width to ensure exact wrapping."""

    wrap_max = 80
    indent = '    '
    wrap_width = wrap_max - len(indent)
    return textwrap.fill(text, width=wrap_width, initial_indent=indent,
                               subsequent_indent=indent, break_long_words=False,
                               break_on_hyphens=False)


def sys_exit(*messages, **kwargs):
    """Performs SystemExit. For modules used from the command line, like
    download and link. To print message, use the same arguments as for
    print_msg()."""

    if messages:
        print_msg(*messages, **kwargs)
    sys.exit(0)
